#include "TH1D.h"
#include "TH2D.h"
#include "THStack.h"
#include "TGraph.h"
#include "TTree.h"
#include "TChain.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TROOT.h"
#include "TStyle.h"
#include "TColor.h"
#include "TLorentzVector.h"
#include "TRandom3.h"
#include "TLegend.h"

#include "RooDataHist.h"
#include "RooRealVar.h"
#include "RooPlot.h"
#include "RooGaussian.h"
#include "RooExponential.h"
#include "RooRealConstant.h"
#include "RooAddPdf.h"
#include "RooFitResult.h"
#include "RooProduct.h"
#include "RooCrystalBall.h"
#include "RooBreitWigner.h"
#include "RooArgSet.h"
#include "RooFFTConvPdf.h"
#include "RooNovosibirsk.h"

#include <string>
#include <iostream>
#include <cmath>

const int nBins = 70;

const Double_t MASS_HIST_MIN = 4500.;
const Double_t MASS_HIST_MAX = 7000.;
const Double_t MASS_HIST_FIT_MIN = 4500.;
const Double_t MASS_HIST_FIT_MAX = 7000.;

void CreateRooFitAndDraw(TH1D *hist, const char *title);

int analysis_fullstream()
{

    std::string title {"SpruceRD_BuToHpMuMu"};

    TChain *data_chain = new TChain("BuToHpMuMu/DecayTree");
    // data_chain->Add("/auto/data/pfeiffer/inclusive_detached_dilepton/data_samples/Collision23_Beam6800GeV-VeloClosed-MagDown-Excl-UT_RealData_Sprucing23r1_90000000_RD.root");
    data_chain->Add("/auto/data/pfeiffer/inclusive_detached_dilepton/data_samples/spruce_magdown_2023_v0r1_tuple_90000000_2023_v0r0p6288631.root");
    // data_chain->Add("/auto/data/pfeiffer/inclusive_detached_dilepton/data_samples/spruce_magdown_2023_v0_tuple_90000000_v0r0p6288631.root");

    Float_t B_BPVFDCHI2,
        B_BPVIPCHI2,
        L1_BPVIPCHI2,
        L2_BPVIPCHI2,
        L1_PT,
        L2_PT,
        Jpsi_BPVFDCHI2,
        Hp_PT,
        Hp_BPVIPCHI2,
        Hp_P;

    Double_t L1_PID_MU,
        L2_PID_MU,
        B_CHI2VXNDOF,
        Jpsi_MAXDOCACHI2,
        Jpsi_CHI2DOF,
        Hp_PID_K,
        Jpsi_M,
        B_M;

    Bool_t L1_ISMUON,
        L2_ISMUON,
        Hlt2RD_BuToKpMuMuDecision,
        Hlt2_InclDetDiMuon_4BodyDecision,
        Hlt2_InclDetDiMuon_3BodyDecision,
        Hlt2_InclDetDiMuonDecision,
        Hlt1TrackMVADecision,
        Hlt1TwoTrackMVADecision;

    data_chain->SetBranchAddress("B_M", &B_M);

    data_chain->SetBranchAddress("B_BPVFDCHI2", &B_BPVFDCHI2);

    data_chain->SetBranchAddress("B_BPVIPCHI2", &B_BPVIPCHI2);

    // data_chain->SetBranchAddress("L1_BPVIPCHI2", &L1_BPVIPCHI2);
    // data_chain->SetBranchAddress("L2_BPVIPCHI2", &L2_BPVIPCHI2);
    // data_chain->SetBranchAddress("L1_PID_MU", &L1_PID_MU);
    // data_chain->SetBranchAddress("L2_PID_MU", &L2_PID_MU);
    // data_chain->SetBranchAddress("L1_ISMUON", &L1_ISMUON);
    // data_chain->SetBranchAddress("L2_ISMUON", &L2_ISMUON);
    // data_chain->SetBranchAddress("L1_PT", &L1_PT);
    // data_chain->SetBranchAddress("L2_PT", &L2_PT);
    // data_chain->SetBranchAddress("Hp_PT", &Hp_PT);
    // data_chain->SetBranchAddress("Hp_BPVIPCHI2", &Hp_BPVIPCHI2);
    // data_chain->SetBranchAddress("Hp_P", &Hp_P);
    // data_chain->SetBranchAddress("Hp_PID_K", &Hp_PID_K);

    data_chain->SetBranchAddress("L1_BPVIPCHI2", &L1_BPVIPCHI2);
    data_chain->SetBranchAddress("L2_BPVIPCHI2", &L2_BPVIPCHI2);
    data_chain->SetBranchAddress("L1_PID_MU", &L1_PID_MU);
    data_chain->SetBranchAddress("L2_PID_MU", &L2_PID_MU);
    data_chain->SetBranchAddress("L1_ISMUON", &L1_ISMUON);
    data_chain->SetBranchAddress("L2_ISMUON", &L2_ISMUON);
    data_chain->SetBranchAddress("L1_PT", &L1_PT);
    data_chain->SetBranchAddress("L2_PT", &L2_PT);
    
    data_chain->SetBranchAddress("B_CHI2VXNDOF", &B_CHI2VXNDOF);
    data_chain->SetBranchAddress("Jpsi_MAXDOCACHI2", &Jpsi_MAXDOCACHI2);
    data_chain->SetBranchAddress("Jpsi_CHI2DOF", &Jpsi_CHI2DOF);

    data_chain->SetBranchAddress("Hp_PT", &Hp_PT);
    data_chain->SetBranchAddress("Hp_BPVIPCHI2", &Hp_BPVIPCHI2);
    data_chain->SetBranchAddress("Hp_P", &Hp_P);
    data_chain->SetBranchAddress("Hp_PID_K", &Hp_PID_K);

    data_chain->SetBranchAddress("Jpsi_BPVFDCHI2", &Jpsi_BPVFDCHI2);
    data_chain->SetBranchAddress("Jpsi_M", &Jpsi_M);

    data_chain->SetBranchAddress("Hlt2RD_BuToKpMuMuDecision", &Hlt2RD_BuToKpMuMuDecision);
    data_chain->SetBranchAddress("Hlt2_InclDetDiMuon_4BodyDecision", &Hlt2_InclDetDiMuon_4BodyDecision);
    data_chain->SetBranchAddress("Hlt2_InclDetDiMuon_3BodyDecision", &Hlt2_InclDetDiMuon_3BodyDecision);
    data_chain->SetBranchAddress("Hlt2_InclDetDiMuonDecision", &Hlt2_InclDetDiMuonDecision);
    data_chain->SetBranchAddress("Hlt1TrackMVADecision", &Hlt1TrackMVADecision);
    data_chain->SetBranchAddress("Hlt1TwoTrackMVADecision", &Hlt1TwoTrackMVADecision);

    TH1D *h1_B_M = new TH1D("h1_B_M", "B Mass", nBins, MASS_HIST_MIN, MASS_HIST_MAX);

    unsigned int entries = data_chain->GetEntries();
    for (unsigned int i = 0; i < entries; i++)
    {
        data_chain->GetEntry(i);

        // if (((B_BPVFDCHI2 > 36) && (B_CHI2VXNDOF < 16) && (B_BPVIPCHI2 < 25) && (Jpsi_MAXDOCACHI2 < 36) && (L1_BPVIPCHI2 > 9) && (L2_BPVIPCHI2 > 9) && (L1_PID_MU > -3) && (L2_PID_MU > -3) && (L1_ISMUON) && (L2_ISMUON) && (L1_PT > 350) && (L2_PT > 350) && (Jpsi_CHI2DOF < 9) && (Jpsi_BPVFDCHI2 > 16) && (Jpsi_M < 5500) && (Hp_PT > 400) && (Hp_BPVIPCHI2 > 6) && (Hp_PT > 400) && (Hp_BPVIPCHI2 > 6) && (Hp_P > 2000) && (Hp_PID_K > -4)) && (Hlt2RD_BuToKpMuMuDecision) && (((Hlt2_InclDetDiMuon_4BodyDecision) || (Hlt2_InclDetDiMuon_3BodyDecision) || (Hlt2_InclDetDiMuonDecision))))
        // {
        //     if (TMath::Abs(Jpsi_M - 3096.9) < 100 && ((Hlt1TrackMVADecision) || (Hlt1TwoTrackMVADecision)))
        //     {
        //         h1_B_M->Fill(B_M);
        //     }
        // }

        if ((((B_BPVFDCHI2 > 36) & (B_CHI2VXNDOF<16) & ( B_BPVIPCHI2<25) & (Jpsi_MAXDOCACHI2 < 36 )&(L1_BPVIPCHI2 > 9) & (L2_BPVIPCHI2 > 9) & (L1_PID_MU > -3) & (L2_PID_MU > -3)& (L1_ISMUON==1) & (L2_ISMUON==1) & (L1_PT>350) & (L2_PT>350) & (Jpsi_CHI2DOF<9 ) & (Jpsi_BPVFDCHI2>16) & (Jpsi_M<5500) & (Hp_PT>400) &( Hp_BPVIPCHI2>6) & (Hp_PT>400) & (Hp_BPVIPCHI2>6) & (Hp_P>2000) & (Hp_PID_K > -4)) & (Hlt2RD_BuToKpMuMuDecision==1)  &(((Hlt2_InclDetDiMuon_4BodyDecision==1) | (Hlt2_InclDetDiMuon_3BodyDecision==1) | (Hlt2_InclDetDiMuonDecision==1))))&(abs(Jpsi_M - 3096.9) < 100) & ((Hlt1TrackMVADecision==1) | (Hlt1TwoTrackMVADecision==1))) 
        {
            h1_B_M->Fill(B_M);
        }

        if (i % 10000 == 0)
        {
            std::cout << "["
                      << "SpruceRD_BuToHpMuMu"
                      << "] Processed event: " << i << " (" << TString::Format("%.2f", ((double)i / (double)entries) * 100.) << "%)" << std::endl;
        }
    }

    TCanvas *c1 = new TCanvas("c1", "c1", 0, 0, 800, 600);
    h1_B_M->Draw();
    c1->Draw();
    c1->SaveAs(TString::Format("images/root_hist_%s_bmass.png", title.c_str()).Data());

    CreateRooFitAndDraw(h1_B_M, title.c_str());

    return 0;
}

void CreateRooFitAndDraw(TH1D *hist, const char *title)
{
    RooRealVar roo_var_mass("var_mass", "B Mass Variable", MASS_HIST_FIT_MIN, MASS_HIST_FIT_MAX);
    roo_var_mass.setRange("fitting_range", MASS_HIST_FIT_MIN, MASS_HIST_FIT_MAX);

    RooDataHist roohist_B_M("roohist_B_M", "B Mass Histogram", roo_var_mass, RooFit::Import(*hist));

    RooPlot *roo_frame_mass = roo_var_mass.frame(RooFit::Title("SpruceRD_BuToHpMuMu"));
    roohist_B_M.plotOn(roo_frame_mass, RooFit::Binning(nBins), RooFit::Name("B Mass Distribution"));
    roo_frame_mass->GetXaxis()->SetTitle("m(#pi^{+}#mu^{+}#mu^{-})");

    TCanvas *c = new TCanvas("roofit_c", "roofit_c", 0, 0, 800, 600);

    roo_frame_mass->Draw();

    TLegend *leg1 = new TLegend(0.65, 0.7, 0.87, 0.8);
    leg1->SetFillColor(kWhite);
    leg1->SetLineColor(kBlack);

    leg1->AddEntry((TObject*)0, TString::Format("Entries: %d", (int)hist->GetEntries()), "");

    leg1->Draw();

    c->Draw();
    c->SaveAs(TString::Format("images/roofit_hist_%s_bmass.png", title).Data());
}